// random number generation class
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef RANDOMNUMBER_H
#define RANDOMNUMBER_H

#include <gsl/gsl_rng.h>
#include <list>

namespace ECSB {

// distribution identifiers
#define GAUSSIAN_DISTRIBUTION	0

// random number generator class
class RNG {
		gsl_rng *rng;  // random number generator
		unsigned long seed;  // generator seed

	public:
		// constructor
		RNG(unsigned long seed = 0);

		// destructor
		~RNG();

		// get generator seed
		unsigned long getSeed(void);

		// generate a uniform random number on the interval [0,1)
		double randomNumber(void);

		// generate a uniform random number on the interval (0,1)
		double randomNumberPos(void);

		// generate a random non-negative integer
		unsigned long randomNonnegativeInteger(void);

		// generate a uniform random non-negative integer
		unsigned long randomNonnegativeInteger(unsigned long);

		// generate a gaussian random number
		double gaussianRandomNumber(double, double);

		// generate a non-negative gaussian random number
		double gaussianRandomNumberNN(double, double);

		// divide integer by two and round up or down randomly
		unsigned long randomHalf(unsigned long);

 		// flip a coin and return true or false
		bool flipCoin(void);

		// generate a random subset from a larger set {min, min+1, ..., max}
		std::list<long> randomSubset(long, long, unsigned long);
};

}

#endif
